@@ -86,6 +86,12 @@ typedef enum {
     APP_X_COLOR
 } ui_drop_target_info;
 
+//TODO: warning: deprecated conversion from string constant to ‘gchar*’
+//
+//Turn out to be warnings that we should probably leave in place. The
+// pointers/types used need to be read-only. So until we correct the using
+// code, those warnings are actually desired. They say "Hey! Fix this". We
+// definitely don't want to hide/ignore them. --JonCruz
 static GtkTargetEntry ui_drop_target_entries [] = {
     {"application/x-color", 0, APP_X_COLOR}
 };
@@ -97,15 +103,18 @@ static guint nui_drop_target_entries = ENTRIES_SIZE(ui_drop_target_entries);
 static Dialog::FillAndStroke *get_fill_and_stroke_panel(SPDesktop *desktop);
 
 SelectedStyle::SelectedStyle(bool /*layout*/)
-    : _desktop (NULL),
+    :
+      current_stroke_width(0),
+
+      _desktop (NULL),
 
       _table(2, 6),
       _fill_label (_("Fill:")),
       _stroke_label (_("Stroke:")),
       _opacity_label (_("O:")),
 
-      _fill_place (SS_FILL),
-      _stroke_place (SS_STROKE),
+      _fill_place(this, SS_FILL),
+      _stroke_place(this, SS_STROKE),
 
       _fill_flag_place (),
       _stroke_flag_place (),
@@ -115,6 +124,7 @@ SelectedStyle::SelectedStyle(bool /*layout*/)
       _opacity_sb (0.02, 0),
 
       _stroke (),
+      _stroke_width_place(this),
       _stroke_width (""),
 
       _opacity_blocked (false),
@@ -375,9 +385,6 @@ SelectedStyle::SelectedStyle(bool /*layout*/)
                      "drag_data_received",
                      G_CALLBACK(dragDataReceived),
                      _drop[SS_FILL]);
-
-    _fill_place.parent = this;
-    _stroke_place.parent = this;
 }
 
 SelectedStyle::~SelectedStyle()
@@ -948,13 +955,13 @@ SelectedStyle::update()
                     _paintserver_id[i] += ")";
 
                     if (SP_IS_LINEARGRADIENT (server)) {
-                        SPGradient *vector = sp_gradient_get_vector(SP_GRADIENT(server), false);
+                        SPGradient *vector = SP_GRADIENT(server)->getVector();
                         sp_gradient_image_set_gradient ((SPGradientImage *) _gradient_preview_l[i], vector);
                         place->add(_gradient_box_l[i]);
                         _tooltips.set_tip(*place, __lgradient[i]);
                         _mode[i] = SS_LGRADIENT;
                     } else if (SP_IS_RADIALGRADIENT (server)) {
-                        SPGradient *vector = sp_gradient_get_vector(SP_GRADIENT(server), false);
+                        SPGradient *vector = SP_GRADIENT(server)->getVector();
                         sp_gradient_image_set_gradient ((SPGradientImage *) _gradient_preview_r[i], vector);
                         place->add(_gradient_box_r[i]);
                         _tooltips.set_tip(*place, __rgradient[i]);
@@ -1038,6 +1045,7 @@ SelectedStyle::update()
     switch (result_sw) {
     case QUERY_STYLE_NOTHING:
         _stroke_width.set_markup("");
+        current_stroke_width = 0;
         break;
     case QUERY_STYLE_SINGLE:
     case QUERY_STYLE_MULTIPLE_AVERAGED:
@@ -1049,6 +1057,8 @@ SelectedStyle::update()
         } else {
             w = query->stroke_width.computed;
         }
+        current_stroke_width = w;
+
         {
             gchar *str = g_strdup_printf(" %.3g", w);
             _stroke_width.set_markup(str);
@@ -1145,12 +1155,18 @@ void SelectedStyle::on_opacity_changed () {
     _opacity_blocked = false;
 }
 
-RotateableSwatch::RotateableSwatch(guint mode) {
-    fillstroke = mode;
-    startcolor_set = false;
-    undokey = "ssrot1";
-    cr = NULL;
-    cr_set = false;
+/* =============================================  RotateableSwatch  */
+
+RotateableSwatch::RotateableSwatch(SelectedStyle *parent, guint mode) :
+    fillstroke(mode),
+    parent(parent),
+    startcolor(0),
+    startcolor_set(false),
+    undokey("ssrot1"),
+    cr(0),
+    cr_set(false)
+
+{
 }
 
 RotateableSwatch::~RotateableSwatch() {
@@ -1236,6 +1252,7 @@ RotateableSwatch::do_motion(double by, guint modifier) {
             g_object_unref (bitmap);
             g_object_unref (mask);
             gdk_window_set_cursor(w->window, cr);
+            gdk_cursor_unref(cr);
             cr_set = true;
         }
     }
@@ -1254,7 +1271,7 @@ RotateableSwatch::do_motion(double by, guint modifier) {
         diff = color_adjust(hsl, by, cc, modifier);
     }
 
-    if (modifier == 3) { // do nothing
+    if (modifier == 3) { // Alt, do nothing
 
     } else if (modifier == 2) { // saturation
         sp_document_maybe_done (sp_desktop_document(parent->getDesktop()), undokey,
@@ -1296,7 +1313,7 @@ RotateableSwatch::do_release(double by, guint modifier) {
         cr_set = false;
     }
 
-    if (modifier == 3) { // nothing
+    if (modifier == 3) { // Alt, do nothing
     } else if (modifier == 2) { // saturation
         sp_document_maybe_done (sp_desktop_document(parent->getDesktop()), undokey,
                                 SP_VERB_DIALOG_FILL_STROKE, ("Adjust saturation"));
@@ -1320,16 +1337,103 @@ RotateableSwatch::do_release(double by, guint modifier) {
     startcolor_set = false;
 }
 
+/* =============================================  RotateableStrokeWidth  */
+
+RotateableStrokeWidth::RotateableStrokeWidth(SelectedStyle *parent) :
+    parent(parent),
+    startvalue(0),
+    startvalue_set(false),
+    undokey("swrot1"),
+    cr(0),
+    cr_set(false)
+{
+}
+
+RotateableStrokeWidth::~RotateableStrokeWidth() {
+}
+
+double
+RotateableStrokeWidth::value_adjust(double current, double by, guint /*modifier*/, bool final)
+{
+    double newval;
+    // by is -1..1
+    if (by < 0) {
+        // map negative 0..-1 to current..0
+        newval = current * (1  +  by);
+    } else {
+        // map positive 0..1 to current..4*current
+        newval = current * (1  +  by) * (1  +  by);
+    }
+
+    SPCSSAttr *css = sp_repr_css_attr_new ();
+    if (final && newval < 1e-6) {
+        // if dragged into zero and this is the final adjust on mouse release, delete stroke;
+        // if it's not final, leave it a chance to increase again (which is not possible with "none")
+        sp_repr_css_set_property (css, "stroke", "none");
+    } else {
+        Inkscape::CSSOStringStream os;
+        os << newval;
+        sp_repr_css_set_property (css, "stroke-width", os.str().c_str());
+    }
+
+    sp_desktop_set_style (parent->getDesktop(), css);
+    sp_repr_css_attr_unref (css);
+    return newval - current;
+}
+
+void
+RotateableStrokeWidth::do_motion(double by, guint modifier) {
+
+    // if this is the first motion after a mouse grab, remember the current width
+    if (!startvalue_set) {
+        startvalue = parent->current_stroke_width;
+        // if it's 0, adjusting (which uses multiplication) will not be able to change it, so we
+        // cheat and provide a non-zero value
+        if (startvalue == 0)
+            startvalue = 1;
+        startvalue_set = true;
+    }
+
+    if (modifier == 3) { // Alt, do nothing
+    } else {
+        double diff = value_adjust(startvalue, by, modifier, false);
+        sp_document_maybe_done (sp_desktop_document(parent->getDesktop()), undokey,
+                                SP_VERB_DIALOG_FILL_STROKE, (_("Adjust stroke width")));
+        parent->getDesktop()->event_context->_message_context->setF(Inkscape::IMMEDIATE_MESSAGE, _("Adjusting <b>stroke width</b>: was %.3g, now <b>%.3g</b> (diff %.3g)"), startvalue, startvalue + diff, diff);
+    }
+}
+
+void
+RotateableStrokeWidth::do_release(double by, guint modifier) {
+
+    if (modifier == 3) { // do nothing
+
+    } else {
+        value_adjust(startvalue, by, modifier, true);
+        startvalue_set = false;
+        sp_document_maybe_done (sp_desktop_document(parent->getDesktop()), undokey,
+                                SP_VERB_DIALOG_FILL_STROKE, (_("Adjust stroke width")));
+    }
+
+    if (!strcmp(undokey, "swrot1")) {
+        undokey = "swrot2";
+    } else {
+        undokey = "swrot1";
+    }
+    parent->getDesktop()->event_context->_message_context->clear();
+}
+
+
 Dialog::FillAndStroke *get_fill_and_stroke_panel(SPDesktop *desktop)
 {
-    if (Dialog::PanelDialogBase *panel_dialog = 
+    if (Dialog::PanelDialogBase *panel_dialog =
         dynamic_cast<Dialog::PanelDialogBase *>(desktop->_dlg_mgr->getDialog("FillAndStroke"))) {
         try {
-            Dialog::FillAndStroke &fill_and_stroke = 
+            Dialog::FillAndStroke &fill_and_stroke =
                 dynamic_cast<Dialog::FillAndStroke &>(panel_dialog->getPanel());
             return &fill_and_stroke;
         } catch (std::exception e) { }
-    }        
+    }
 
     return 0;
 }
