@@ -59,9 +59,14 @@ class SelectedStyle;
 class RotateableSwatch: public Rotateable 
 {
 public:
-    RotateableSwatch(guint mode);
+    RotateableSwatch(SelectedStyle *parent, guint mode);
     ~RotateableSwatch();
 
+    double color_adjust (float *hsl, double by, guint32 cc, guint state);
+    virtual void do_motion (double by, guint state);
+    virtual void do_release (double by, guint state);
+
+private:
     guint fillstroke;
 
     SelectedStyle *parent;
@@ -69,14 +74,32 @@ public:
     guint32 startcolor;
     bool startcolor_set;
 
-    gchar *undokey;
+    gchar const *undokey;
 
     GdkCursor *cr;
     bool cr_set;
+};
 
-    double color_adjust (float *hsl, double by, guint32 cc, guint state);
+class RotateableStrokeWidth: public Rotateable 
+{
+public:
+    RotateableStrokeWidth(SelectedStyle *parent);
+    ~RotateableStrokeWidth();
+
+    double value_adjust(double current, double by, guint modifier, bool final);
     virtual void do_motion (double by, guint state);
     virtual void do_release (double by, guint state);
+
+private:
+    SelectedStyle *parent;
+
+    double startvalue;
+    bool startvalue_set;
+
+    gchar const *undokey;
+
+    GdkCursor *cr;
+    bool cr_set;
 };
 
 class SelectedStyle : public Gtk::HBox
@@ -95,6 +118,8 @@ public:
 
     guint _mode[2];
 
+    double current_stroke_width;
+
 protected:
     SPDesktop *_desktop;
 
@@ -150,7 +175,7 @@ protected:
     Glib::ustring __multiple[2];
 
     Gtk::HBox _stroke;
-    Gtk::EventBox _stroke_width_place;
+    RotateableStrokeWidth _stroke_width_place;
     Gtk::Label _stroke_width;
 
     Glib::ustring _paintserver_id[2];
